#!/usr/bin/python

# gobject-foo.py: generate standard GObject type macros etc.
#
# The master copy of this program is in the telepathy-glib repository -
# please make any changes there.
#
# Copyright (C) 2007 Collabora Ltd. <http://www.collabora.co.uk/>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

def gobject_header(head, tail, as_interface=False):
    out = []
    o = out.append

    name = head + '_' + tail
    MixedCase = name.replace('_', '')
    lower_case = name.lower()
    UPPER_CASE = name.upper()

    gtype = head.upper() + '_TYPE_' + tail.upper()

    o("typedef struct _%s %s;" % (MixedCase, MixedCase))
    o("typedef struct _%sClass %sClass;" % (MixedCase, MixedCase))
    o("typedef struct _%sPrivate %sPrivate;" % (MixedCase, MixedCase))
    o("")
    o("GType %s_get_type (void);" % lower_case)
    o("")

    o("#define %s \\" % gtype)
    o("  (%s_get_type ())" % lower_case)

    o("#define %s(obj) \\" % UPPER_CASE)
    o("  (G_TYPE_CHECK_INSTANCE_CAST ((obj), %s, \\" % gtype)
    o("                               %s))" % MixedCase)

    if not as_interface:
        o("#define %s_CLASS(klass) \\" % UPPER_CASE)
        o("  (G_TYPE_CHECK_CLASS_CAST ((klass), %s, \\" % gtype)
        o("                            %sClass))" % MixedCase)

    o("#define %s_IS_%s(obj) \\" % (head.upper(), tail.upper()))
    o("  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), %s))" % gtype)

    if not as_interface:
        o("#define %s_IS_%s_CLASS(klass) \\" % (head.upper(), tail.upper()))
        o("  (G_TYPE_CHECK_CLASS_TYPE ((klass), %s))" % gtype)

    o("#define %s_GET_CLASS(obj) \\" % UPPER_CASE)
    o("  (G_TYPE_INSTANCE_GET_CLASS ((obj), %s, \\" % gtype)
    o("                              %sClass))" % MixedCase)

    return out

if __name__ == '__main__':
    import sys
    from getopt import gnu_getopt

    options, argv = gnu_getopt(sys.argv[1:], '', ['interface'])

    as_interface = False

    for opt, val in options:
        if opt == '--interface':
            as_interface = True

    head, tail = argv

    print '\n'.join(gobject_header(head, tail, as_interface=as_interface))




"""
Infrastructure code for testing Ring
"""

import os
import sys
import dbus
import servicetest
import time
from servicetest import (unwrap, Event)
from twisted.internet import reactor

def install_colourer():
    def red(s):
        return '\x1b[31m%s\x1b[0m' % s

    def green(s):
        return '\x1b[32m%s\x1b[0m' % s

    patterns = {
        'handled': green,
        'not handled': red,
        }

    class Colourer:
        def __init__(self, fh, patterns):
            self.fh = fh
            self.patterns = patterns

        def write(self, s):
            f = self.patterns.get(s, lambda x: x)
            self.fh.write(f(s))

    sys.stdout = Colourer(sys.stdout, patterns)
    return sys.stdout

class Simulator(object):
    def __init__(self):
        self.bus = dbus.SystemBus()

        try:
            manager = dbus.Interface(self.bus.get_object('org.ofono', '/'),
                dbus_interface='org.ofono.Manager')
        except dbus.exceptions.DBusException:
              print "  Ofono needs to be running to execute tests"
              os._exit(1)

def make_connection(bus, event_func, params=None):
    default_params = {
        'modem': dbus.ObjectPath('/phonesim'),
        }

    if params:
        default_params.update(params)

    return servicetest.make_connection(bus, event_func, 'ring', 'tel',
        default_params)

def exec_test_deferred (funs, params, protocol=None, timeout=None):
    colourer = None

    if sys.stdout.isatty():
        colourer = install_colourer()

    queue = servicetest.IteratingEventQueue(timeout)
    queue.verbose = (
        os.environ.get('CHECK_TWISTED_VERBOSE', '') != ''
        or '-v' in sys.argv)

    bus = dbus.SessionBus()

    sim = Simulator()

    bus.add_signal_receiver(
        lambda *args, **kw:
            queue.append(
                Event('dbus-signal',
                    path=unwrap(kw['path']),
                    signal=kw['member'], args=map(unwrap, args),
                    interface=kw['interface'])),
        None,       # signal name
        None,       # interface
        None,
        path_keyword='path',
        member_keyword='member',
        interface_keyword='interface',
        byte_arrays=True
        )

    try:
        for f in funs:
            conn = make_connection(bus, queue.append, params)
            f(queue, bus, conn)
    except Exception:
        import traceback
        traceback.print_exc()

    try:
        if colourer:
          sys.stdout = colourer.fh
        reactor.crash()

        # force Disconnect in case the test crashed and didn't disconnect
        # properly.  We need to call this async because the BaseIRCServer
        # class must do something in response to the Disconnect call and if we
        # call it synchronously, we're blocking ourself from responding to the
        # quit method.
        servicetest.call_async(queue, conn, 'Disconnect')

        if 'RING_TEST_REFDBG' in os.environ:
            # we have to wait for the timeout so the process is properly
            # exited and refdbg can generate its report
            time.sleep(5.5)

    except dbus.DBusException:
        pass

def exec_tests(funs, params=None, protocol=None, timeout=None):
  reactor.callWhenRunning (exec_test_deferred, funs, params, protocol, timeout)
  reactor.run()

def exec_test(fun, params=None, protocol=None, timeout=None):
  exec_tests([fun], params, protocol, timeout)




#!/usr/bin/python

from sys import argv, stdout, stderr
import xml.dom.minidom

from libglibcodegen import NS_TP, get_docstring, \
        get_descendant_text, get_by_path

class Generator(object):
    def __init__(self, prefix, implfile, declfile, dom):
        self.prefix = prefix + '_'

        assert declfile.endswith('.h')
        docfile = declfile[:-2] + '-gtk-doc.h'

        self.impls = open(implfile, 'w')
        self.decls = open(declfile, 'w')
        self.docs = open(docfile, 'w')
        self.spec = get_by_path(dom, "spec")[0]

    def h(self, code):
        self.decls.write(code.encode('utf-8'))

    def c(self, code):
        self.impls.write(code.encode('utf-8'))

    def d(self, code):
        self.docs.write(code.encode('utf-8'))

    def __call__(self):
        for f in self.h, self.c:
            self.do_header(f)
        self.do_body()

    # Header
    def do_header(self, f):
        f('/* Generated from: ')
        f(get_descendant_text(get_by_path(self.spec, 'title')))
        version = get_by_path(self.spec, "version")
        if version:
            f(' version ' + get_descendant_text(version))
        f('\n\n')
        for copyright in get_by_path(self.spec, 'copyright'):
            f(get_descendant_text(copyright))
            f('\n')
        f('\n')
        f(get_descendant_text(get_by_path(self.spec, 'license')))
        f(get_descendant_text(get_by_path(self.spec, 'docstring')))
        f("""
 */

""")

    # Body
    def do_body(self):
        for iface in self.spec.getElementsByTagName('interface'):
            self.do_iface(iface)

    def do_iface(self, iface):
        parent_name = get_by_path(iface, '../@name')
        self.d("""\
/**
 * %(IFACE_DEFINE)s:
 *
 * The interface name "%(name)s"
 */
""" % {'IFACE_DEFINE' : (self.prefix + 'IFACE_' + \
            parent_name).upper().replace('/', ''),
       'name' : iface.getAttribute('name')})

        self.h("""
#define %(IFACE_DEFINE)s \\
"%(name)s"
""" % {'IFACE_DEFINE' : (self.prefix + 'IFACE_' + \
            parent_name).upper().replace('/', ''),
       'name' : iface.getAttribute('name')})

        self.d("""
/**
 * %(IFACE_QUARK_DEFINE)s:
 *
 * Expands to a call to a function that returns a quark for the interface \
name "%(name)s"
 */
""" % {'IFACE_QUARK_DEFINE' : (self.prefix + 'IFACE_QUARK_' + \
            parent_name).upper().replace('/', ''),
       'iface_quark_func' : (self.prefix + 'iface_quark_' + \
            parent_name).lower().replace('/', ''),
       'name' : iface.getAttribute('name')})

        self.h("""
#define %(IFACE_QUARK_DEFINE)s \\
  (%(iface_quark_func)s ())

GQuark %(iface_quark_func)s (void);

""" % {'IFACE_QUARK_DEFINE' : (self.prefix + 'IFACE_QUARK_' + \
            parent_name).upper().replace('/', ''),
       'iface_quark_func' : (self.prefix + 'iface_quark_' + \
            parent_name).lower().replace('/', ''),
       'name' : iface.getAttribute('name')})

        self.c("""\
GQuark
%(iface_quark_func)s (void)
{
  static GQuark quark = 0;

  if (G_UNLIKELY (quark == 0))
    {
      quark = g_quark_from_static_string ("%(name)s");
    }

  return quark;
}

""" % {'iface_quark_func' : (self.prefix + 'iface_quark_' + \
            parent_name).lower().replace('/', ''),
       'name' : iface.getAttribute('name')})

        for prop in iface.getElementsByTagNameNS(None, 'property'):
            self.d("""
/**
 * %(IFACE_PREFIX)s_%(PROP_UC)s:
 *
 * The fully-qualified property name "%(name)s.%(prop)s"
 */
""" % {'IFACE_PREFIX' : (self.prefix + 'PROP_' + \
                parent_name).upper().replace('/', ''),
           'PROP_UC': prop.getAttributeNS(NS_TP, "name-for-bindings").upper(),
           'name' : iface.getAttribute('name'),
           'prop' : prop.getAttribute('name'),
           })

            self.h("""
#define %(IFACE_PREFIX)s_%(PROP_UC)s \\
"%(name)s.%(prop)s"
""" % {'IFACE_PREFIX' : (self.prefix + 'PROP_' + \
                parent_name).upper().replace('/', ''),
           'PROP_UC': prop.getAttributeNS(NS_TP, "name-for-bindings").upper(),
           'name' : iface.getAttribute('name'),
           'prop' : prop.getAttribute('name'),
           })


        for prop in iface.getElementsByTagNameNS(NS_TP, 'contact-attribute'):
            self.d("""
/**
 * %(TOKEN_PREFIX)s_%(TOKEN_UC)s:
 *
 * The fully-qualified contact attribute token name "%(name)s/%(prop)s"
 */
""" % {'TOKEN_PREFIX' : (self.prefix + 'TOKEN_' + \
                parent_name).upper().replace('/', ''),
           'TOKEN_UC': prop.getAttributeNS(None, "name").upper().replace("-", "_").replace(".", "_"),
           'name' : iface.getAttribute('name'),
           'prop' : prop.getAttribute('name'),
           })

            self.h("""
#define %(TOKEN_PREFIX)s_%(TOKEN_UC)s \\
"%(name)s/%(prop)s"
""" % {'TOKEN_PREFIX' : (self.prefix + 'TOKEN_' + \
                parent_name).upper().replace('/', ''),
           'TOKEN_UC': prop.getAttributeNS(None, "name").upper().replace("-", "_").replace(".", "_"),
           'name' : iface.getAttribute('name'),
           'prop' : prop.getAttribute('name'),
           })

        for prop in iface.getElementsByTagNameNS(NS_TP, 'hct'):
            if (prop.getAttribute('is-family') != "yes"):
                self.d("""
/**
 * %(TOKEN_PREFIX)s_%(TOKEN_UC)s:
 *
 * The fully-qualified capability token name "%(name)s/%(prop)s"
 */
""" % {'TOKEN_PREFIX' : (self.prefix + 'TOKEN_' + \
                parent_name).upper().replace('/', ''),
           'TOKEN_UC': prop.getAttributeNS(None, "name").upper().replace("-", "_").replace(".", "_"),
           'name' : iface.getAttribute('name'),
           'prop' : prop.getAttribute('name'),
           })

                self.h("""
#define %(TOKEN_PREFIX)s_%(TOKEN_UC)s \\
"%(name)s/%(prop)s"
""" % {'TOKEN_PREFIX' : (self.prefix + 'TOKEN_' + \
                parent_name).upper().replace('/', ''),
           'TOKEN_UC': prop.getAttributeNS(None, "name").upper().replace("-", "_").replace(".", "_"),
           'name' : iface.getAttribute('name'),
           'prop' : prop.getAttribute('name'),
           })

if __name__ == '__main__':
    argv = argv[1:]
    Generator(argv[0], argv[1], argv[2], xml.dom.minidom.parse(argv[3]))()



# Copyright (C) 2009 Nokia Corporation
# Copyright (C) 2009-2013 Collabora Ltd.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301 USA

"""
Infrastructure code for testing Telepathy services.
"""

from twisted.internet import glib2reactor
from twisted.internet.protocol import Protocol, Factory, ClientFactory
glib2reactor.install()
import sys
import time
import os

import pprint
import unittest

import dbus
import dbus.lowlevel
from dbus.mainloop.glib import DBusGMainLoop
DBusGMainLoop(set_as_default=True)

from twisted.internet import reactor

import constants as cs

tp_name_prefix = cs.PREFIX
tp_path_prefix = cs.PATH_PREFIX

class DictionarySupersetOf (object):
    """Utility class for expecting "a dictionary with at least these keys"."""
    def __init__(self, dictionary):
        self._dictionary = dictionary
    def __repr__(self):
        return "DictionarySupersetOf(%s)" % self._dictionary
    def __eq__(self, other):
        """would like to just do:
        return set(other.items()).issuperset(self._dictionary.items())
        but it turns out that this doesn't work if you have another dict
        nested in the values of your dicts"""
        try:
            for k,v in self._dictionary.items():
                if k not in other or other[k] != v:
                    return False
            return True
        except TypeError: # other is not iterable
            return False

class Event(object):
    def __init__(self, type, **kw):
        self.__dict__.update(kw)
        self.type = type
        (self.subqueue, self.subtype) = type.split ("-", 1)

    def __str__(self):
        return '\n'.join([ str(type(self)) ] + format_event(self))

def format_event(event):
    ret = ['- type %s' % event.type]

    for key in sorted(dir(event)):
        if key != 'type' and not key.startswith('_'):
            ret.append('- %s: %s' % (
                key, pprint.pformat(getattr(event, key))))

            if key == 'error':
                ret.append('%s' % getattr(event, key))

    return ret

class EventPattern:
    def __init__(self, type, **properties):
        self.type = type
        self.predicate = None
        if 'predicate' in properties:
            self.predicate = properties['predicate']
            del properties['predicate']
        self.properties = properties
        (self.subqueue, self.subtype) = type.split ("-", 1)

    def __repr__(self):
        properties = dict(self.properties)

        if self.predicate is not None:
            properties['predicate'] = self.predicate

        return '%s(%r, **%r)' % (
            self.__class__.__name__, self.type, properties)

    def match(self, event):
        if event.type != self.type:
            return False

        for key, value in self.properties.iteritems():
            try:
                if getattr(event, key) != value:
                    return False
            except AttributeError:
                return False

        if self.predicate is None or self.predicate(event):
            return True

        return False


class TimeoutError(Exception):
    pass

class ForbiddenEventOccurred(Exception):
    def __init__(self, event):
        Exception.__init__(self)
        self.event = event

    def __str__(self):
        return '\n' + '\n'.join(format_event(self.event))

class BaseEventQueue:
    """Abstract event queue base class.

    Implement the wait() method to have something that works.
    """

    def __init__(self, timeout=None):
        self.verbose = False
        self.forbidden_events = set()
        self.event_queues = {}

        if timeout is None:
            self.timeout = 5
        else:
            self.timeout = timeout

    def log(self, s):
        if self.verbose:
            print s

    def log_queues(self, queues):
        self.log ("Waiting for event on: %s" % ", ".join(queues))

    def log_event(self, event):
        self.log('got event:')

        if self.verbose:
            map(self.log, format_event(event))

    def forbid_events(self, patterns):
        """
        Add patterns (an iterable of EventPattern) to the set of forbidden
        events. If a forbidden event occurs during an expect or expect_many,
        the test will fail.
        """
        self.forbidden_events.update(set(patterns))

    def unforbid_events(self, patterns):
        """
        Remove 'patterns' (an iterable of EventPattern) from the set of
        forbidden events. These must be the same EventPattern pointers that
        were passed to forbid_events.
        """
        self.forbidden_events.difference_update(set(patterns))

    def unforbid_all(self):
        """
        Remove all patterns from the set of forbidden events.
        """
        self.forbidden_events.clear()

    def _check_forbidden(self, event):
        for e in self.forbidden_events:
            if e.match(event):
                raise ForbiddenEventOccurred(event)

    def expect(self, type, **kw):
        """
        Waits for an event matching the supplied pattern to occur, and returns
        it. For example, to await a D-Bus signal with particular arguments:

            e = q.expect('dbus-signal', signal='Badgers', args=["foo", 42])
        """
        pattern = EventPattern(type, **kw)
        t = time.time()

        while True:
            try:
                event = self.wait([pattern.subqueue])
            except TimeoutError:
                self.log('timeout')
                self.log('still expecting:')
                self.log(' - %r' % pattern)
                raise

            self._check_forbidden(event)

            if pattern.match(event):
                self.log('handled, took %0.3f ms'
                    % ((time.time() - t) * 1000.0) )
                self.log('')
                return event

            self.log('not handled')
            self.log('')

    def expect_many(self, *patterns):
        """
        Waits for events matching all of the supplied EventPattern instances to
        return, and returns a list of events in the same order as the patterns
        they matched. After a pattern is successfully matched, it is not
        considered for future events; if more than one unsatisfied pattern
        matches an event, the first "wins".

        Note that the expected events may occur in any order. If you're
        expecting a series of events in a particular order, use repeated calls
        to expect() instead.

        This method is useful when you're awaiting a number of events which may
        happen in any order. For instance, in telepathy-gabble, calling a D-Bus
        method often causes a value to be returned immediately, as well as a
        query to be sent to the server. Since these events may reach the test
        in either order, the following is incorrect and will fail if the IQ
        happens to reach the test first:

            ret = q.expect('dbus-return', method='Foo')
            query = q.expect('stream-iq', query_ns=ns.FOO)

        The following would be correct:

            ret, query = q.expect_many(
                EventPattern('dbus-return', method='Foo'),
                EventPattern('stream-iq', query_ns=ns.FOO),
            )
        """
        ret = [None] * len(patterns)
        t = time.time()

        while None in ret:
            try:
                queues = set()
                for i, pattern in enumerate(patterns):
                    if ret[i] is None:
                        queues.add(pattern.subqueue)
                event = self.wait(queues)
            except TimeoutError:
                self.log('timeout')
                self.log('still expecting:')
                for i, pattern in enumerate(patterns):
                    if ret[i] is None:
                        self.log(' - %r' % pattern)
                raise
            self._check_forbidden(event)

            for i, pattern in enumerate(patterns):
                if ret[i] is None and pattern.match(event):
                    self.log('handled, took %0.3f ms'
                        % ((time.time() - t) * 1000.0) )
                    self.log('')
                    ret[i] = event
                    break
            else:
                self.log('not handled')
                self.log('')

        return ret

    def demand(self, type, **kw):
        pattern = EventPattern(type, **kw)

        event = self.wait([pattern.subqueue])

        if pattern.match(event):
            self.log('handled')
            self.log('')
            return event

        self.log('not handled')
        raise RuntimeError('expected %r, got %r' % (pattern, event))

    def queues_available(self, queues):
        if queues == None:
            return self.event_queues.keys()
        else:
            available = self.event_queues.keys()
            return filter(lambda x: x in available, queues)


    def pop_next(self, queue):
        events = self.event_queues[queue]
        e = events.pop(0)
        if not events:
           self.event_queues.pop (queue)
        return e

    def append(self, event):
        self.log ("Adding to queue")
        self.log_event (event)
        self.event_queues[event.subqueue] = \
            self.event_queues.get(event.subqueue, []) + [event]

class IteratingEventQueue(BaseEventQueue):
    """Event queue that works by iterating the Twisted reactor."""

    def __init__(self, timeout=None):
        BaseEventQueue.__init__(self, timeout)
        self._dbus_method_impls = []
        self._buses = []
        # a message filter which will claim we handled everything
        self._dbus_dev_null = \
                lambda bus, message: dbus.lowlevel.HANDLER_RESULT_HANDLED

    def wait(self, queues=None):
        stop = [False]

        def later():
            stop[0] = True

        delayed_call = reactor.callLater(self.timeout, later)

        self.log_queues(queues)

        qa = self.queues_available(queues)
        while not qa and (not stop[0]):
            reactor.iterate(0.01)
            qa = self.queues_available(queues)

        if qa:
            delayed_call.cancel()
            e = self.pop_next (qa[0])
            self.log_event (e)
            return e
        else:
            raise TimeoutError

    def add_dbus_method_impl(self, cb, bus=None, **kwargs):
        if bus is None:
            bus = self._buses[0]

        self._dbus_method_impls.append(
                (EventPattern('dbus-method-call', **kwargs), cb))

    def dbus_emit(self, path, iface, name, *a, **k):
        bus = k.pop('bus', self._buses[0])
        assert 'signature' in k, k
        message = dbus.lowlevel.SignalMessage(path, iface, name)
        message.append(*a, **k)
        bus.send_message(message)

    def dbus_return(self, in_reply_to, *a, **k):
        bus = k.pop('bus', self._buses[0])
        assert 'signature' in k, k
        reply = dbus.lowlevel.MethodReturnMessage(in_reply_to)
        reply.append(*a, **k)
        bus.send_message(reply)

    def dbus_raise(self, in_reply_to, name, message=None, bus=None):
        if bus is None:
            bus = self._buses[0]

        reply = dbus.lowlevel.ErrorMessage(in_reply_to, name, message)
        bus.send_message(reply)

    def attach_to_bus(self, bus):
        if not self._buses:
            # first-time setup
            self._dbus_filter_bound_method = self._dbus_filter

        self._buses.append(bus)

        # Only subscribe to messages on the first bus connection (assumed to
        # be the shared session bus connection used by the simulated connection
        # manager and most of the test suite), not on subsequent bus
        # connections (assumed to represent extra clients).
        #
        # When we receive a method call on the other bus connections, ignore
        # it - the eavesdropping filter installed on the first bus connection
        # will see it too.
        #
        # This is highly counter-intuitive, but it means our messages are in
        # a guaranteed order (we don't have races between messages arriving on
        # various connections).
        if len(self._buses) > 1:
            bus.add_message_filter(self._dbus_dev_null)
            return

        try:
            # for dbus > 1.5
            bus.add_match_string("eavesdrop=true,type='signal'")
        except dbus.DBusException:
            bus.add_match_string("type='signal'")
            bus.add_match_string("type='method_call'")
        else:
            bus.add_match_string("eavesdrop=true,type='method_call'")

        bus.add_message_filter(self._dbus_filter_bound_method)

        bus.add_signal_receiver(
                lambda *args, **kw:
                    self.append(
                        Event('dbus-signal',
                            path=unwrap(kw['path']),
                            signal=kw['member'],
                            args=map(unwrap, args),
                            interface=kw['interface'])),
                None,
                None,
                None,
                path_keyword='path',
                member_keyword='member',
                interface_keyword='interface',
                byte_arrays=True,
                )

    def cleanup(self):
        if self._buses:
            self._buses[0].remove_message_filter(self._dbus_filter_bound_method)
        for bus in self._buses[1:]:
            bus.remove_message_filter(self._dbus_dev_null)

        self._buses = []
        self._dbus_method_impls = []

    def _dbus_filter(self, bus, message):
        if isinstance(message, dbus.lowlevel.MethodCallMessage):

            destination = message.get_destination()
            sender = message.get_sender()

            if (destination == 'org.freedesktop.DBus' or
                    sender == self._buses[0].get_unique_name()):
                # suppress reply and don't make an Event
                return dbus.lowlevel.HANDLER_RESULT_HANDLED

            e = Event('dbus-method-call', message=message,
                interface=message.get_interface(), path=message.get_path(),
                raw_args=message.get_args_list(byte_arrays=True),
                args=map(unwrap, message.get_args_list(byte_arrays=True)),
                destination=str(destination),
                method=message.get_member(),
                sender=message.get_sender(),
                handled=False)

            for pair in self._dbus_method_impls:
                pattern, cb = pair
                if pattern.match(e):
                    cb(e)
                    e.handled = True
                    break

            self.append(e)

            return dbus.lowlevel.HANDLER_RESULT_HANDLED

        return dbus.lowlevel.HANDLER_RESULT_NOT_YET_HANDLED

class TestEventQueue(BaseEventQueue):
    def __init__(self, events):
        BaseEventQueue.__init__(self)
        for e in events:
            self.append (e)

    def wait(self, queues = None):
        qa = self.queues_available(queues)

        if qa:
            return self.pop_next (qa[0])
        else:
            raise TimeoutError

class EventQueueTest(unittest.TestCase):
    def test_expect(self):
        queue = TestEventQueue([Event('test-foo'), Event('test-bar')])
        assert queue.expect('test-foo').type == 'test-foo'
        assert queue.expect('test-bar').type == 'test-bar'

    def test_expect_many(self):
        queue = TestEventQueue([Event('test-foo'),
            Event('test-bar')])
        bar, foo = queue.expect_many(
            EventPattern('test-bar'),
            EventPattern('test-foo'))
        assert bar.type == 'test-bar'
        assert foo.type == 'test-foo'

    def test_expect_many2(self):
        # Test that events are only matched against patterns that haven't yet
        # been matched. This tests a regression.
        queue = TestEventQueue([Event('test-foo', x=1), Event('test-foo', x=2)])
        foo1, foo2 = queue.expect_many(
            EventPattern('test-foo'),
            EventPattern('test-foo'))
        assert foo1.type == 'test-foo' and foo1.x == 1
        assert foo2.type == 'test-foo' and foo2.x == 2

    def test_expect_queueing(self):
        queue = TestEventQueue([Event('foo-test', x=1),
            Event('foo-test', x=2)])

        queue.append(Event('bar-test', x=1))
        queue.append(Event('bar-test', x=2))

        queue.append(Event('baz-test', x=1))
        queue.append(Event('baz-test', x=2))

        for x in xrange(1,2):
            e = queue.expect ('baz-test')
            assertEquals (x, e.x)

            e = queue.expect ('bar-test')
            assertEquals (x, e.x)

            e = queue.expect ('foo-test')
            assertEquals (x, e.x)

    def test_timeout(self):
        queue = TestEventQueue([])
        self.assertRaises(TimeoutError, queue.expect, 'test-foo')

    def test_demand(self):
        queue = TestEventQueue([Event('test-foo'), Event('test-bar')])
        foo = queue.demand('test-foo')
        assert foo.type == 'test-foo'

    def test_demand_fail(self):
        queue = TestEventQueue([Event('test-foo'), Event('test-bar')])
        self.assertRaises(RuntimeError, queue.demand, 'test-bar')

def unwrap(x):
    """Hack to unwrap D-Bus values, so that they're easier to read when
    printed."""

    if isinstance(x, list):
        return map(unwrap, x)

    if isinstance(x, tuple):
        return tuple(map(unwrap, x))

    if isinstance(x, dict):
        return dict([(unwrap(k), unwrap(v)) for k, v in x.iteritems()])

    if isinstance(x, dbus.Boolean):
        return bool(x)

    for t in [unicode, str, long, int, float]:
        if isinstance(x, t):
            return t(x)

    return x

def call_async(test, proxy, method, *args, **kw):
    """Call a D-Bus method asynchronously and generate an event for the
    resulting method return/error."""

    def reply_func(*ret):
        test.append(Event('dbus-return', method=method,
            value=unwrap(ret)))

    def error_func(err):
        test.append(Event('dbus-error', method=method, error=err,
            name=err.get_dbus_name(), message=str(err)))

    method_proxy = getattr(proxy, method)
    kw.update({'reply_handler': reply_func, 'error_handler': error_func})
    method_proxy(*args, **kw)

def sync_dbus(bus, q, proxy):
    # Dummy D-Bus method call. We can't use DBus.Peer.Ping() because libdbus
    # replies to that message immediately, rather than handing it up to
    # dbus-glib and thence the application, which means that Ping()ing the
    # application doesn't ensure that it's processed all D-Bus messages prior
    # to our ping.
    call_async(q, dbus.Interface(proxy, 'org.freedesktop.Telepathy.Tests'),
        'DummySyncDBus')
    q.expect('dbus-error', method='DummySyncDBus')

class ProxyWrapper:
    def __init__(self, object, default, others={}):
        self.object = object
        self.default_interface = dbus.Interface(object, default)
        self.Properties = dbus.Interface(object, dbus.PROPERTIES_IFACE)
        self.TpProperties = \
            dbus.Interface(object, tp_name_prefix + '.Properties')
        self.interfaces = dict([
            (name, dbus.Interface(object, iface))
            for name, iface in others.iteritems()])

    def __getattr__(self, name):
        if name in self.interfaces:
            return self.interfaces[name]

        if name in self.object.__dict__:
            return getattr(self.object, name)

        return getattr(self.default_interface, name)

class ConnWrapper(ProxyWrapper):
    def inspect_contact_sync(self, handle):
        return self.inspect_contacts_sync([handle])[0]

    def inspect_contacts_sync(self, handles):
        h2asv = self.Contacts.GetContactAttributes(handles, [], True)
        ret = []
        for h in handles:
            ret.append(h2asv[h][cs.ATTR_CONTACT_ID])
        return ret

    def get_contact_handle_sync(self, identifier):
        return self.Contacts.GetContactByID(identifier, [])[0]

    def get_contact_handles_sync(self, ids):
        return [self.get_contact_handle_sync(i) for i in ids]

def wrap_connection(conn):
    return ConnWrapper(conn, tp_name_prefix + '.Connection',
        dict([
            (name, tp_name_prefix + '.Connection.Interface.' + name)
            for name in ['Aliasing', 'Avatars', 'Capabilities', 'Contacts',
              'SimplePresence', 'Requests']] +
        [('Peer', 'org.freedesktop.DBus.Peer'),
         ('ContactCapabilities', cs.CONN_IFACE_CONTACT_CAPS),
         ('ContactInfo', cs.CONN_IFACE_CONTACT_INFO),
         ('Location', cs.CONN_IFACE_LOCATION),
         ('Future', tp_name_prefix + '.Connection.FUTURE'),
         ('MailNotification', cs.CONN_IFACE_MAIL_NOTIFICATION),
         ('ContactList', cs.CONN_IFACE_CONTACT_LIST),
         ('ContactGroups', cs.CONN_IFACE_CONTACT_GROUPS),
         ('ContactBlocking', cs.CONN_IFACE_CONTACT_BLOCKING),
         ('PowerSaving', cs.CONN_IFACE_POWER_SAVING),
         ('Addressing', cs.CONN_IFACE_ADDRESSING),
         ('ClientTypes', cs.CONN_IFACE_CLIENT_TYPES),
        ]))

class ChannelWrapper(ProxyWrapper):
    def send_msg_sync(self, txt):
        message = [
                { 'message-type': cs.MT_NORMAL, },
                { 'content-type': 'text/plain',
                  'content': txt
                }]
        self.Messages.SendMessage(message, 0)

def wrap_channel(chan, type_, extra=None):
    interfaces = {
        type_: tp_name_prefix + '.Channel.Type.' + type_,
        'Channel': cs.CHANNEL,
        'Group': cs.CHANNEL_IFACE_GROUP,
        'Hold': cs.CHANNEL_IFACE_HOLD,
        'Messages': cs.CHANNEL_IFACE_MESSAGES,
        'RoomConfig1': cs.CHANNEL_IFACE_ROOM_CONFIG,
        'ChatState': cs.CHANNEL_IFACE_CHAT_STATE,
        'Destroyable': cs.CHANNEL_IFACE_DESTROYABLE,
        'Password': cs.CHANNEL_IFACE_PASSWORD,
        }

    if extra:
        interfaces.update(dict([
            (name, tp_name_prefix + '.Channel.Interface.' + name)
            for name in extra]))

    return ChannelWrapper(chan, tp_name_prefix + '.Channel', interfaces)


def wrap_content(chan, extra=None):
    interfaces = {
        'DTMF': cs.CALL_CONTENT_IFACE_DTMF,
        'Media': cs.CALL_CONTENT_IFACE_MEDIA,
        }

    if extra:
        interfaces.update(dict([
            (name, tp_name_prefix + '.Call1.Content.Interface.' + name)
            for name in extra]))

    return ProxyWrapper(chan, tp_name_prefix + '.Call1.Content', interfaces)

def make_connection(bus, event_func, name, proto, params):
    cm = bus.get_object(
        tp_name_prefix + '.ConnectionManager.%s' % name,
        tp_path_prefix + '/ConnectionManager/%s' % name,
        introspect=False)
    cm_iface = dbus.Interface(cm, tp_name_prefix + '.ConnectionManager')

    connection_name, connection_path = cm_iface.RequestConnection(
        proto, dbus.Dictionary(params, signature='sv'))
    conn = wrap_connection(bus.get_object(connection_name, connection_path))

    return conn

def make_channel_proxy(conn, path, iface):
    bus = dbus.SessionBus()
    chan = bus.get_object(conn.object.bus_name, path)
    chan = dbus.Interface(chan, tp_name_prefix + '.' + iface)
    return chan

# block_reading can be used if the test want to choose when we start to read
# data from the socket.
class EventProtocol(Protocol):
    def __init__(self, queue=None, block_reading=False):
        self.queue = queue
        self.block_reading = block_reading

    def dataReceived(self, data):
        if self.queue is not None:
            self.queue.append(Event('socket-data', protocol=self,
                data=data))

    def sendData(self, data):
        self.transport.write(data)

    def connectionMade(self):
        if self.block_reading:
            self.transport.stopReading()

    def connectionLost(self, reason=None):
        if self.queue is not None:
            self.queue.append(Event('socket-disconnected', protocol=self))

class EventProtocolFactory(Factory):
    def __init__(self, queue, block_reading=False):
        self.queue = queue
        self.block_reading = block_reading

    def _create_protocol(self):
        return EventProtocol(self.queue, self.block_reading)

    def buildProtocol(self, addr):
        proto = self._create_protocol()
        self.queue.append(Event('socket-connected', protocol=proto))
        return proto

class EventProtocolClientFactory(EventProtocolFactory, ClientFactory):
    pass

def watch_tube_signals(q, tube):
    def got_signal_cb(*args, **kwargs):
        q.append(Event('tube-signal',
            path=kwargs['path'],
            signal=kwargs['member'],
            args=map(unwrap, args),
            tube=tube))

    tube.add_signal_receiver(got_signal_cb,
        path_keyword='path', member_keyword='member',
        byte_arrays=True)

def pretty(x):
    return pprint.pformat(unwrap(x))

def assertEquals(expected, value):
    if expected != value:
        raise AssertionError(
            "expected:\n%s\ngot:\n%s" % (pretty(expected), pretty(value)))

def assertSameSets(expected, value):
    exp_set = set(expected)
    val_set = set(value)

    if exp_set != val_set:
        raise AssertionError(
            "expected contents:\n%s\ngot:\n%s" % (
                pretty(exp_set), pretty(val_set)))

def assertNotEquals(expected, value):
    if expected == value:
        raise AssertionError(
            "expected something other than:\n%s" % pretty(value))

def assertContains(element, value):
    if element not in value:
        raise AssertionError(
            "expected:\n%s\nin:\n%s" % (pretty(element), pretty(value)))

def assertDoesNotContain(element, value):
    if element in value:
        raise AssertionError(
            "expected:\n%s\nnot in:\n%s" % (pretty(element), pretty(value)))

def assertLength(length, value):
    if len(value) != length:
        raise AssertionError("expected: length %d, got length %d:\n%s" % (
            length, len(value), pretty(value)))

def assertFlagsSet(flags, value):
    masked = value & flags
    if masked != flags:
        raise AssertionError(
            "expected flags %u, of which only %u are set in %u" % (
            flags, masked, value))

def assertFlagsUnset(flags, value):
    masked = value & flags
    if masked != 0:
        raise AssertionError(
            "expected none of flags %u, but %u are set in %u" % (
            flags, masked, value))

def assertDBusError(name, error):
    if error.get_dbus_name() != name:
        raise AssertionError(
            "expected DBus error named:\n  %s\ngot:\n  %s\n(with message: %s)"
            % (name, error.get_dbus_name(), error.message))

def install_colourer():
    def red(s):
        return '\x1b[31m%s\x1b[0m' % s

    def green(s):
        return '\x1b[32m%s\x1b[0m' % s

    patterns = {
        'handled': green,
        'not handled': red,
        }

    class Colourer:
        def __init__(self, fh, patterns):
            self.fh = fh
            self.patterns = patterns

        def write(self, s):
            for p, f in self.patterns.items():
                if s.startswith(p):
                    self.fh.write(f(p) + s[len(p):])
                    return

            self.fh.write(s)

    sys.stdout = Colourer(sys.stdout, patterns)
    return sys.stdout

# this is just to shut up unittest.
class DummyStream(object):
    def write(self, s):
        if 'CHECK_TWISTED_VERBOSE' in os.environ:
            print s,

    def flush(self):
        pass

if __name__ == '__main__':
    stream = DummyStream()
    runner = unittest.TextTestRunner(stream=stream)
    unittest.main(testRunner=runner)



#!/usr/bin/python

# glib-ginterface-gen.py: service-side interface generator
#
# Generate dbus-glib 0.x service GInterfaces from the Telepathy specification.
# The master copy of this program is in the telepathy-glib repository -
# please make any changes there.
#
# Copyright (C) 2006, 2007 Collabora Limited
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import sys
import os.path
import xml.dom.minidom

from libglibcodegen import Signature, type_to_gtype, cmp_by_name, \
        NS_TP, dbus_gutils_wincaps_to_uscore, \
        signal_to_marshal_name, method_to_glue_marshal_name


NS_TP = "http://telepathy.freedesktop.org/wiki/DbusSpec#extensions-v0"

class Generator(object):

    def __init__(self, dom, prefix, basename, signal_marshal_prefix,
                 headers, end_headers, not_implemented_func,
                 allow_havoc):
        self.dom = dom
        self.__header = []
        self.__body = []
        self.__docs = []

        assert prefix.endswith('_')
        assert not signal_marshal_prefix.endswith('_')

        # The main_prefix, sub_prefix thing is to get:
        # FOO_ -> (FOO_, _)
        # FOO_SVC_ -> (FOO_, _SVC_)
        # but
        # FOO_BAR/ -> (FOO_BAR_, _)
        # FOO_BAR/SVC_ -> (FOO_BAR_, _SVC_)

        if '/' in prefix:
            main_prefix, sub_prefix = prefix.upper().split('/', 1)
            prefix = prefix.replace('/', '_')
        else:
            main_prefix, sub_prefix = prefix.upper().split('_', 1)

        self.MAIN_PREFIX_ = main_prefix + '_'
        self._SUB_PREFIX_ = '_' + sub_prefix

        self.Prefix_ = prefix
        self.Prefix = prefix.replace('_', '')
        self.prefix_ = prefix.lower()
        self.PREFIX_ = prefix.upper()

        self.basename = basename
        self.signal_marshal_prefix = signal_marshal_prefix
        self.headers = headers
        self.end_headers = end_headers
        self.not_implemented_func = not_implemented_func
        self.allow_havoc = allow_havoc

    def h(self, s):
        if isinstance(s, unicode):
            s = s.encode('utf-8')
        self.__header.append(s)

    def b(self, s):
        if isinstance(s, unicode):
            s = s.encode('utf-8')
        self.__body.append(s)

    def d(self, s):
        if isinstance(s, unicode):
            s = s.encode('utf-8')
        self.__docs.append(s)

    def do_node(self, node):
        node_name = node.getAttribute('name').replace('/', '')
        node_name_mixed = self.node_name_mixed = node_name.replace('_', '')
        node_name_lc = self.node_name_lc = node_name.lower()
        node_name_uc = self.node_name_uc = node_name.upper()

        interfaces = node.getElementsByTagName('interface')
        assert len(interfaces) == 1, interfaces
        interface = interfaces[0]
        self.iface_name = interface.getAttribute('name')

        tmp = interface.getAttribute('tp:implement-service')
        if tmp == "no":
            return

        tmp = interface.getAttribute('tp:causes-havoc')
        if tmp and not self.allow_havoc:
            raise AssertionError('%s is %s' % (self.iface_name, tmp))

        self.b('static const DBusGObjectInfo _%s%s_object_info;'
               % (self.prefix_, node_name_lc))
        self.b('')

        methods = interface.getElementsByTagName('method')
        signals = interface.getElementsByTagName('signal')
        properties = interface.getElementsByTagName('property')
        # Don't put properties in dbus-glib glue
        glue_properties = []

        self.b('struct _%s%sClass {' % (self.Prefix, node_name_mixed))
        self.b('    GTypeInterface parent_class;')
        for method in methods:
            self.b('    %s %s;' % self.get_method_impl_names(method))
        self.b('};')
        self.b('')

        if signals:
            self.b('enum {')
            for signal in signals:
                self.b('    %s,' % self.get_signal_const_entry(signal))
            self.b('    N_%s_SIGNALS' % node_name_uc)
            self.b('};')
            self.b('static guint %s_signals[N_%s_SIGNALS] = {0};'
                   % (node_name_lc, node_name_uc))
            self.b('')

        self.b('static void %s%s_base_init (gpointer klass);'
               % (self.prefix_, node_name_lc))
        self.b('')

        self.b('GType')
        self.b('%s%s_get_type (void)'
               % (self.prefix_, node_name_lc))
        self.b('{')
        self.b('  static GType type = 0;')
        self.b('')
        self.b('  if (G_UNLIKELY (type == 0))')
        self.b('    {')
        self.b('      static const GTypeInfo info = {')
        self.b('        sizeof (%s%sClass),' % (self.Prefix, node_name_mixed))
        self.b('        %s%s_base_init, /* base_init */'
               % (self.prefix_, node_name_lc))
        self.b('        NULL, /* base_finalize */')
        self.b('        NULL, /* class_init */')
        self.b('        NULL, /* class_finalize */')
        self.b('        NULL, /* class_data */')
        self.b('        0,')
        self.b('        0, /* n_preallocs */')
        self.b('        NULL /* instance_init */')
        self.b('      };')
        self.b('')
        self.b('      type = g_type_register_static (G_TYPE_INTERFACE,')
        self.b('          "%s%s", &info, 0);' % (self.Prefix, node_name_mixed))
        self.b('    }')
        self.b('')
        self.b('  return type;')
        self.b('}')
        self.b('')

        self.d('/**')
        self.d(' * %s%s:' % (self.Prefix, node_name_mixed))
        self.d(' *')
        self.d(' * Dummy typedef representing any implementation of this '
               'interface.')
        self.d(' */')

        self.h('typedef struct _%s%s %s%s;'
               % (self.Prefix, node_name_mixed, self.Prefix, node_name_mixed))
        self.h('')

        self.d('/**')
        self.d(' * %s%sClass:' % (self.Prefix, node_name_mixed))
        self.d(' *')
        self.d(' * The class of %s%s.' % (self.Prefix, node_name_mixed))

        if methods:
            self.d(' *')
            self.d(' * In a full implementation of this interface (i.e. all')
            self.d(' * methods implemented), the interface initialization')
            self.d(' * function used in G_IMPLEMENT_INTERFACE() would')
            self.d(' * typically look like this:')
            self.d(' *')
            self.d(' * <programlisting>')
            self.d(' * static void')
            self.d(' * implement_%s (gpointer klass,' % self.node_name_lc)
            self.d(' *     gpointer unused G_GNUC_UNUSED)')
            self.d(' * {')
            self.d(' * #define IMPLEMENT(x) %s%s_implement_&num;&num;x (\\'
                   % (self.prefix_, self.node_name_lc))
            self.d(' *   klass, my_object_&num;&num;x)')

            for method in methods:
                class_member_name = method.getAttribute('tp:name-for-bindings')
                class_member_name = class_member_name.lower()
                self.d(' *   IMPLEMENT (%s);' % class_member_name)

            self.d(' * #undef IMPLEMENT')
            self.d(' * }')
            self.d(' * </programlisting>')
        else:
            self.d(' * This interface has no D-Bus methods, so an')
            self.d(' * implementation can typically pass %NULL to')
            self.d(' * G_IMPLEMENT_INTERFACE() as the interface')
            self.d(' * initialization function.')

        self.d(' */')
        self.d('')

        self.h('typedef struct _%s%sClass %s%sClass;'
               % (self.Prefix, node_name_mixed, self.Prefix, node_name_mixed))
        self.h('')
        self.h('GType %s%s_get_type (void);'
               % (self.prefix_, node_name_lc))

        gtype = self.current_gtype = \
                self.MAIN_PREFIX_ + 'TYPE' + self._SUB_PREFIX_ + node_name_uc
        classname = self.Prefix + node_name_mixed

        self.h('#define %s \\\n  (%s%s_get_type ())'
               % (gtype, self.prefix_, node_name_lc))
        self.h('#define %s%s(obj) \\\n'
               '  (G_TYPE_CHECK_INSTANCE_CAST((obj), %s, %s))'
               % (self.PREFIX_, node_name_uc, gtype, classname))
        self.h('#define %sIS%s%s(obj) \\\n'
               '  (G_TYPE_CHECK_INSTANCE_TYPE((obj), %s))'
               % (self.MAIN_PREFIX_, self._SUB_PREFIX_, node_name_uc, gtype))
        self.h('#define %s%s_GET_CLASS(obj) \\\n'
               '  (G_TYPE_INSTANCE_GET_INTERFACE((obj), %s, %sClass))'
               % (self.PREFIX_, node_name_uc, gtype, classname))
        self.h('')
        self.h('')

        base_init_code = []

        for method in methods:
            self.do_method(method)

        for signal in signals:
            base_init_code.extend(self.do_signal(signal))

        self.b('static inline void')
        self.b('%s%s_base_init_once (gpointer klass G_GNUC_UNUSED)'
               % (self.prefix_, node_name_lc))
        self.b('{')

        if properties:
            self.b('  static TpDBusPropertiesMixinPropInfo properties[%d] = {'
                   % (len(properties) + 1))

            for m in properties:
                access = m.getAttribute('access')
                assert access in ('read', 'write', 'readwrite')

                if access == 'read':
                    flags = 'TP_DBUS_PROPERTIES_MIXIN_FLAG_READ'
                elif access == 'write':
                    flags = 'TP_DBUS_PROPERTIES_MIXIN_FLAG_WRITE'
                else:
                    flags = ('TP_DBUS_PROPERTIES_MIXIN_FLAG_READ | '
                             'TP_DBUS_PROPERTIES_MIXIN_FLAG_WRITE')

                self.b('      { 0, %s, "%s", 0, NULL, NULL }, /* %s */'
                       % (flags, m.getAttribute('type'), m.getAttribute('name')))

            self.b('      { 0, 0, NULL, 0, NULL, NULL }')
            self.b('  };')
            self.b('  static TpDBusPropertiesMixinIfaceInfo interface =')
            self.b('      { 0, properties, NULL, NULL };')
            self.b('')


        self.b('  dbus_g_object_type_install_info (%s%s_get_type (),'
               % (self.prefix_, node_name_lc))
        self.b('      &_%s%s_object_info);'
               % (self.prefix_, node_name_lc))
        self.b('')

        if properties:
            self.b('  interface.dbus_interface = g_quark_from_static_string '
                   '("%s");' % self.iface_name)

            for i, m in enumerate(properties):
                self.b('  properties[%d].name = g_quark_from_static_string ("%s");'
                       % (i, m.getAttribute('name')))
                self.b('  properties[%d].type = %s;'
                           % (i, type_to_gtype(m.getAttribute('type'))[1]))

            self.b('  tp_svc_interface_set_dbus_properties_info (%s, &interface);'
                   % self.current_gtype)

            self.b('')

        for s in base_init_code:
            self.b(s)
        self.b('}')

        self.b('static void')
        self.b('%s%s_base_init (gpointer klass)'
               % (self.prefix_, node_name_lc))
        self.b('{')
        self.b('  static gboolean initialized = FALSE;')
        self.b('')
        self.b('  if (!initialized)')
        self.b('    {')
        self.b('      initialized = TRUE;')
        self.b('      %s%s_base_init_once (klass);'
               % (self.prefix_, node_name_lc))
        self.b('    }')
        # insert anything we need to do per implementation here
        self.b('}')

        self.h('')

        self.b('static const DBusGMethodInfo _%s%s_methods[] = {'
               % (self.prefix_, node_name_lc))

        method_blob, offsets = self.get_method_glue(methods)

        for method, offset in zip(methods, offsets):
            self.do_method_glue(method, offset)

        if len(methods) == 0:
            # empty arrays are a gcc extension, so put in a dummy member
            self.b("  { NULL, NULL, 0 }")

        self.b('};')
        self.b('')

        self.b('static const DBusGObjectInfo _%s%s_object_info = {'
               % (self.prefix_, node_name_lc))
        self.b('  0,')  # version
        self.b('  _%s%s_methods,' % (self.prefix_, node_name_lc))
        self.b('  %d,' % len(methods))
        self.b('"' + method_blob.replace('\0', '\\0') + '",')
        self.b('"' + self.get_signal_glue(signals).replace('\0', '\\0') + '",')
        self.b('"' +
               self.get_property_glue(glue_properties).replace('\0', '\\0') +
               '",')
        self.b('};')
        self.b('')

        self.node_name_mixed = None
        self.node_name_lc = None
        self.node_name_uc = None

    def get_method_glue(self, methods):
        info = []
        offsets = []

        for method in methods:
            offsets.append(len(''.join(info)))

            info.append(self.iface_name + '\0')
            info.append(method.getAttribute('name') + '\0')

            info.append('A\0')    # async

            counter = 0
            for arg in method.getElementsByTagName('arg'):
                out = arg.getAttribute('direction') == 'out'

                name = arg.getAttribute('name')
                if not name:
                    assert out
                    name = 'arg%u' % counter
                counter += 1

                info.append(name + '\0')

                if out:
                    info.append('O\0')
                else:
                    info.append('I\0')

                if out:
                    info.append('F\0')    # not const
                    info.append('N\0')    # not error or return
                info.append(arg.getAttribute('type') + '\0')

            info.append('\0')

        return ''.join(info) + '\0', offsets

    def do_method_glue(self, method, offset):
        lc_name = method.getAttribute('tp:name-for-bindings')
        if method.getAttribute('name') != lc_name.replace('_', ''):
            raise AssertionError('Method %s tp:name-for-bindings (%s) does '
                    'not match' % (method.getAttribute('name'), lc_name))
        lc_name = lc_name.lower()

        marshaller = method_to_glue_marshal_name(method,
                self.signal_marshal_prefix)
        wrapper = self.prefix_ + self.node_name_lc + '_' + lc_name

        self.b("  { (GCallback) %s, %s, %d }," % (wrapper, marshaller, offset))

    def get_signal_glue(self, signals):
        info = []

        for signal in signals:
            info.append(self.iface_name)
            info.append(signal.getAttribute('name'))

        return '\0'.join(info) + '\0\0'

    # the implementation can be the same
    get_property_glue = get_signal_glue

    def get_method_impl_names(self, method):
        dbus_method_name = method.getAttribute('name')

        class_member_name = method.getAttribute('tp:name-for-bindings')
        if dbus_method_name != class_member_name.replace('_', ''):
            raise AssertionError('Method %s tp:name-for-bindings (%s) does '
                    'not match' % (dbus_method_name, class_member_name))
        class_member_name = class_member_name.lower()

        stub_name = (self.prefix_ + self.node_name_lc + '_' +
                     class_member_name)
        return (stub_name + '_impl', class_member_name + '_cb')

    def do_method(self, method):
        assert self.node_name_mixed is not None

        in_class = []

        # Examples refer to Thing.DoStuff (su) -> ii

        # DoStuff
        dbus_method_name = method.getAttribute('name')
        # do_stuff
        class_member_name = method.getAttribute('tp:name-for-bindings')
        if dbus_method_name != class_member_name.replace('_', ''):
            raise AssertionError('Method %s tp:name-for-bindings (%s) does '
                    'not match' % (dbus_method_name, class_member_name))
        class_member_name = class_member_name.lower()

        # void tp_svc_thing_do_stuff (TpSvcThing *, const char *, guint,
        #   DBusGMethodInvocation *);
        stub_name = (self.prefix_ + self.node_name_lc + '_' +
                     class_member_name)
        # typedef void (*tp_svc_thing_do_stuff_impl) (TpSvcThing *,
        #   const char *, guint, DBusGMethodInvocation);
        impl_name = stub_name + '_impl'
        # void tp_svc_thing_return_from_do_stuff (DBusGMethodInvocation *,
        #   gint, gint);
        ret_name = (self.prefix_ + self.node_name_lc + '_return_from_' +
                    class_member_name)

        # Gather arguments
        in_args = []
        out_args = []
        for i in method.getElementsByTagName('arg'):
            name = i.getAttribute('name')
            direction = i.getAttribute('direction') or 'in'
            dtype = i.getAttribute('type')

            assert direction in ('in', 'out')

            if name:
                name = direction + '_' + name
            elif direction == 'in':
                name = direction + str(len(in_args))
            else:
                name = direction + str(len(out_args))

            ctype, gtype, marshaller, pointer = type_to_gtype(dtype)

            if pointer:
                ctype = 'const ' + ctype

            struct = (ctype, name)

            if direction == 'in':
                in_args.append(struct)
            else:
                out_args.append(struct)

        # Implementation type declaration (in header, docs separated)
        self.d('/**')
        self.d(' * %s:' % impl_name)
        self.d(' * @self: The object implementing this interface')
        for (ctype, name) in in_args:
            self.d(' * @%s: %s (FIXME, generate documentation)'
                   % (name, ctype))
        self.d(' * @context: Used to return values or throw an error')
        self.d(' *')
        self.d(' * The signature of an implementation of the D-Bus method')
        self.d(' * %s on interface %s.' % (dbus_method_name, self.iface_name))
        self.d(' */')

        self.h('typedef void (*%s) (%s%s *self,'
          % (impl_name, self.Prefix, self.node_name_mixed))
        for (ctype, name) in in_args:
            self.h('    %s%s,' % (ctype, name))
        self.h('    DBusGMethodInvocation *context);')

        # Class member (in class definition)
        in_class.append('    %s %s;' % (impl_name, class_member_name))

        # Stub definition (in body only - it's static)
        self.b('static void')
        self.b('%s (%s%s *self,'
           % (stub_name, self.Prefix, self.node_name_mixed))
        for (ctype, name) in in_args:
            self.b('    %s%s,' % (ctype, name))
        self.b('    DBusGMethodInvocation *context)')
        self.b('{')
        self.b('  %s impl = (%s%s_GET_CLASS (self)->%s_cb);'
          % (impl_name, self.PREFIX_, self.node_name_uc, class_member_name))
        self.b('')
        self.b('  if (impl != NULL)')
        tmp = ['self'] + [name for (ctype, name) in in_args] + ['context']
        self.b('    {')
        self.b('      (impl) (%s);' % ',\n        '.join(tmp))
        self.b('    }')
        self.b('  else')
        self.b('    {')
        if self.not_implemented_func:
            self.b('      %s (context);' % self.not_implemented_func)
        else:
            self.b('      GError e = { DBUS_GERROR, ')
            self.b('           DBUS_GERROR_UNKNOWN_METHOD,')
            self.b('           "Method not implemented" };')
            self.b('')
            self.b('      dbus_g_method_return_error (context, &e);')
        self.b('    }')
        self.b('}')
        self.b('')

        # Implementation registration (in both header and body)
        self.h('void %s%s_implement_%s (%s%sClass *klass, %s impl);'
               % (self.prefix_, self.node_name_lc, class_member_name,
                  self.Prefix, self.node_name_mixed, impl_name))

        self.d('/**')
        self.d(' * %s%s_implement_%s:'
               % (self.prefix_, self.node_name_lc, class_member_name))
        self.d(' * @klass: A class whose instances implement this interface')
        self.d(' * @impl: A callback used to implement the %s D-Bus method'
               % dbus_method_name)
        self.d(' *')
        self.d(' * Register an implementation for the %s method in the vtable'
               % dbus_method_name)
        self.d(' * of an implementation of this interface. To be called from')
        self.d(' * the interface init function.')
        self.d(' */')

        self.b('void')
        self.b('%s%s_implement_%s (%s%sClass *klass, %s impl)'
               % (self.prefix_, self.node_name_lc, class_member_name,
                  self.Prefix, self.node_name_mixed, impl_name))
        self.b('{')
        self.b('  klass->%s_cb = impl;' % class_member_name)
        self.b('}')
        self.b('')

        # Return convenience function (static inline, in header)
        self.d('/**')
        self.d(' * %s:' % ret_name)
        self.d(' * @context: The D-Bus method invocation context')
        for (ctype, name) in out_args:
            self.d(' * @%s: %s (FIXME, generate documentation)'
                   % (name, ctype))
        self.d(' *')
        self.d(' * Return successfully by calling dbus_g_method_return().')
        self.d(' * This inline function exists only to provide type-safety.')
        self.d(' */')
        self.d('')

        tmp = (['DBusGMethodInvocation *context'] +
               [ctype + name for (ctype, name) in out_args])
        self.h('static inline')
        self.h('/* this comment is to stop gtkdoc realising this is static */')
        self.h(('void %s (' % ret_name) + (',\n    '.join(tmp)) + ');')
        self.h('static inline void')
        self.h(('%s (' % ret_name) + (',\n    '.join(tmp)) + ')')
        self.h('{')
        tmp = ['context'] + [name for (ctype, name) in out_args]
        self.h('  dbus_g_method_return (' + ',\n      '.join(tmp) + ');')
        self.h('}')
        self.h('')

        return in_class

    def get_signal_const_entry(self, signal):
        assert self.node_name_uc is not None
        return ('SIGNAL_%s_%s'
                % (self.node_name_uc, signal.getAttribute('name')))

    def do_signal(self, signal):
        assert self.node_name_mixed is not None

        in_base_init = []

        # for signal: Thing::StuffHappened (s, u)
        # we want to emit:
        # void tp_svc_thing_emit_stuff_happened (gpointer instance,
        #    const char *arg0, guint arg1);

        dbus_name = signal.getAttribute('name')

        ugly_name = signal.getAttribute('tp:name-for-bindings')
        if dbus_name != ugly_name.replace('_', ''):
            raise AssertionError('Signal %s tp:name-for-bindings (%s) does '
                    'not match' % (dbus_name, ugly_name))

        stub_name = (self.prefix_ + self.node_name_lc + '_emit_' +
                     ugly_name.lower())

        const_name = self.get_signal_const_entry(signal)

        # Gather arguments
        args = []
        for i in signal.getElementsByTagName('arg'):
            name = i.getAttribute('name')
            dtype = i.getAttribute('type')
            tp_type = i.getAttribute('tp:type')

            if name:
                name = 'arg_' + name
            else:
                name = 'arg' + str(len(args))

            ctype, gtype, marshaller, pointer = type_to_gtype(dtype)

            if pointer:
                ctype = 'const ' + ctype

            struct = (ctype, name, gtype)
            args.append(struct)

        tmp = (['gpointer instance'] +
               [ctype + name for (ctype, name, gtype) in args])

        self.h(('void %s (' % stub_name) + (',\n    '.join(tmp)) + ');')

        # FIXME: emit docs

        self.d('/**')
        self.d(' * %s:' % stub_name)
        self.d(' * @instance: The object implementing this interface')
        for (ctype, name, gtype) in args:
            self.d(' * @%s: %s (FIXME, generate documentation)'
                   % (name, ctype))
        self.d(' *')
        self.d(' * Type-safe wrapper around g_signal_emit to emit the')
        self.d(' * %s signal on interface %s.'
               % (dbus_name, self.iface_name))
        self.d(' */')

        self.b('void')
        self.b(('%s (' % stub_name) + (',\n    '.join(tmp)) + ')')
        self.b('{')
        self.b('  g_assert (instance != NULL);')
        self.b('  g_assert (G_TYPE_CHECK_INSTANCE_TYPE (instance, %s));'
               % (self.current_gtype))
        tmp = (['instance', '%s_signals[%s]' % (self.node_name_lc, const_name),
                '0'] + [name for (ctype, name, gtype) in args])
        self.b('  g_signal_emit (' + ',\n      '.join(tmp) + ');')
        self.b('}')
        self.b('')

        signal_name = dbus_gutils_wincaps_to_uscore(dbus_name).replace('_',
                '-')

        self.d('/**')
        self.d(' * %s%s::%s:'
                % (self.Prefix, self.node_name_mixed, signal_name))
        for (ctype, name, gtype) in args:
            self.d(' * @%s: %s (FIXME, generate documentation)'
                   % (name, ctype))
        self.d(' *')
        self.d(' * The %s D-Bus signal is emitted whenever '
                'this GObject signal is.' % dbus_name)
        self.d(' */')
        self.d('')

        in_base_init.append('  %s_signals[%s] ='
                            % (self.node_name_lc, const_name))
        in_base_init.append('  g_signal_new ("%s",' % signal_name)
        in_base_init.append('      G_OBJECT_CLASS_TYPE (klass),')
        in_base_init.append('      G_SIGNAL_RUN_LAST|G_SIGNAL_DETAILED,')
        in_base_init.append('      0,')
        in_base_init.append('      NULL, NULL,')
        in_base_init.append('      %s,'
                % signal_to_marshal_name(signal, self.signal_marshal_prefix))
        in_base_init.append('      G_TYPE_NONE,')
        tmp = ['%d' % len(args)] + [gtype for (ctype, name, gtype) in args]
        in_base_init.append('      %s);' % ',\n      '.join(tmp))
        in_base_init.append('')

        return in_base_init

    def have_properties(self, nodes):
        for node in nodes:
            interface =  node.getElementsByTagName('interface')[0]
            if interface.getElementsByTagName('property'):
                return True
        return False

    def __call__(self):
        nodes = self.dom.getElementsByTagName('node')
        nodes.sort(cmp_by_name)

        self.h('#include <glib-object.h>')
        self.h('#include <dbus/dbus-glib.h>')

        if self.have_properties(nodes):
            self.h('#include <telepathy-glib/dbus-properties-mixin.h>')

        self.h('')
        self.h('G_BEGIN_DECLS')
        self.h('')

        self.b('#include "%s.h"' % self.basename)
        self.b('')
        for header in self.headers:
            self.b('#include %s' % header)
        self.b('')

        for node in nodes:
            self.do_node(node)

        self.h('')
        self.h('G_END_DECLS')

        self.b('')
        for header in self.end_headers:
            self.b('#include %s' % header)

        self.h('')
        self.b('')
        open(self.basename + '.h', 'w').write('\n'.join(self.__header))
        open(self.basename + '.c', 'w').write('\n'.join(self.__body))
        open(self.basename + '-gtk-doc.h', 'w').write('\n'.join(self.__docs))


def cmdline_error():
    print """\
usage:
    gen-ginterface [OPTIONS] xmlfile Prefix_
options:
    --include='<header.h>' (may be repeated)
    --include='"header.h"' (ditto)
    --include-end='"header.h"' (ditto)
        Include extra headers in the generated .c file
    --signal-marshal-prefix='prefix'
        Use the given prefix on generated signal marshallers (default is
        prefix.lower()).
    --filename='BASENAME'
        Set the basename for the output files (default is prefix.lower()
        + 'ginterfaces')
    --not-implemented-func='symbol'
        Set action when methods not implemented in the interface vtable are
        called. symbol must have signature
            void symbol (DBusGMethodInvocation *context)
        and return some sort of "not implemented" error via
            dbus_g_method_return_error (context, ...)
"""
    sys.exit(1)


if __name__ == '__main__':
    from getopt import gnu_getopt

    options, argv = gnu_getopt(sys.argv[1:], '',
                               ['filename=', 'signal-marshal-prefix=',
                                'include=', 'include-end=',
                                'allow-unstable',
                                'not-implemented-func='])

    try:
        prefix = argv[1]
    except IndexError:
        cmdline_error()

    basename = prefix.lower() + 'ginterfaces'
    signal_marshal_prefix = prefix.lower().rstrip('_')
    headers = []
    end_headers = []
    not_implemented_func = ''
    allow_havoc = False

    for option, value in options:
        if option == '--filename':
            basename = value
        elif option == '--signal-marshal-prefix':
            signal_marshal_prefix = value
        elif option == '--include':
            if value[0] not in '<"':
                value = '"%s"' % value
            headers.append(value)
        elif option == '--include-end':
            if value[0] not in '<"':
                value = '"%s"' % value
            end_headers.append(value)
        elif option == '--not-implemented-func':
            not_implemented_func = value
        elif option == '--allow-unstable':
            allow_havoc = True

    try:
        dom = xml.dom.minidom.parse(argv[0])
    except IndexError:
        cmdline_error()

    Generator(dom, prefix, basename, signal_marshal_prefix, headers,
              end_headers, not_implemented_func, allow_havoc)()



#!/usr/bin/python

import sys
import xml.dom.minidom

from libglibcodegen import NS_TP, camelcase_to_upper, get_docstring, \
        get_descendant_text

class Generator(object):
    def __init__(self, dom):
        self.dom = dom
        self.errors = self.dom.getElementsByTagNameNS(NS_TP, 'errors')[0]

    def do_header(self):
        print '/* Generated from the Telepathy spec\n'
        copyrights = self.errors.getElementsByTagNameNS(NS_TP, 'copyright')
        for copyright in copyrights:
            print get_descendant_text(copyright)
        license = self.errors.getElementsByTagNameNS(NS_TP, 'license')[0]
        print '\n' + get_descendant_text(license) + '\n*/'

    def do_enum_values(self):
        for error in self.errors.getElementsByTagNameNS(NS_TP, 'error'):
            print ''
            nick = error.getAttribute('name').replace(' ', '')
            name = camelcase_to_upper(nick.replace('.', ''))
            ns = error.parentNode.getAttribute('namespace')
            enum = 'TP_ERROR_' + name
            print '        /* ' + ns + '.' + name
            print '    ' + get_docstring(error)
            print '     */'
            print '        { %s, "%s", "%s" },' % (enum, enum, nick)


    def do_get_type(self):
        print """
#include <_gen/telepathy-errors.h>

GType
tp_error_get_type (void)
{
  static GType etype = 0;
  if (G_UNLIKELY (etype == 0))
    {
      static const GEnumValue values[] = {"""
        self.do_enum_values()
        print """\
      };

      etype = g_enum_register_static ("TpError", values);
    }
  return etype;
}
"""

    def __call__(self):
        self.do_header()
        self.do_get_type()

if __name__ == '__main__':
    argv = sys.argv[1:]
    Generator(xml.dom.minidom.parse(argv[0]))()




"""
Test connecting to a server.
"""

from ringtest import exec_test
from servicetest import EventPattern, call_async
import constants as cs

def test(q, bus, conn):
    conn.Connect()
    q.expect('dbus-signal', signal='StatusChanged', args=[cs.CONN_STATUS_CONNECTING, cs.CSR_REQUESTED])
    q.expect('dbus-signal', signal='StatusChanged', args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED])

    call_async(q, conn, 'Disconnect')
    q.expect_many(
            EventPattern('dbus-signal', signal='StatusChanged', args=[2, cs.CSR_REQUESTED]),
            EventPattern('dbus-return', method='Disconnect'))

if __name__ == '__main__':
    exec_test(test)




"""
Test Ring's o.fd.T.Protocol implementation
"""

import dbus
from servicetest import unwrap, tp_path_prefix, assertEquals, assertContains
from ringtest import exec_test
import constants as cs

def test(q, bus, conn):
    cm = bus.get_object(cs.CM + '.ring',
        tp_path_prefix + '/ConnectionManager/ring')
    cm_iface = dbus.Interface(cm, cs.CM)
    cm_prop_iface = dbus.Interface(cm, cs.PROPERTIES_IFACE)

    protocols = unwrap(cm_prop_iface.Get(cs.CM, 'Protocols'))
    assertEquals(set(['tel']), set(protocols.keys()))

    protocol_names = unwrap(cm_iface.ListProtocols())
    assertEquals(set(['tel']), set(protocol_names))

    cm_params = cm_iface.GetParameters('tel')
    local_props = protocols['tel']
    local_params = local_props[cs.PROTOCOL + '.Parameters']
    assertEquals(cm_params, local_params)

    proto = bus.get_object(cm.bus_name, cm.object_path + '/tel')
    proto_prop_iface = dbus.Interface(proto, cs.PROPERTIES_IFACE)
    proto_props = unwrap(proto_prop_iface.GetAll(cs.PROTOCOL))

    for key in ['Parameters', 'Interfaces', 'ConnectionInterfaces',
      'RequestableChannelClasses', u'VCardField', u'EnglishName', u'Icon']:
        a = local_props[cs.PROTOCOL + '.' + key]
        b = proto_props[key]
        assertEquals(a, b)

    assertEquals('tel', proto_props['VCardField'])
    assertEquals('Mobile Telephony', proto_props['EnglishName'])
    assertEquals('im-tel', proto_props['Icon'])

    assertContains(cs.CONN_IFACE_REQUESTS, proto_props['ConnectionInterfaces'])

if __name__ == '__main__':
    exec_test(test)



#!/usr/bin/python

import sys
import xml.dom.minidom

from libglibcodegen import NS_TP, camelcase_to_upper, get_docstring, \
        get_descendant_text

class Generator(object):
    def __init__(self, dom):
        self.dom = dom
        self.errors = self.dom.getElementsByTagNameNS(NS_TP, 'errors')[0]

    def do_header(self):
        print '/* Generated from the Telepathy spec\n'
        copyrights = self.errors.getElementsByTagNameNS(NS_TP, 'copyright')
        for copyright in copyrights:
            print get_descendant_text(copyright)
        license = self.errors.getElementsByTagNameNS(NS_TP, 'license')[0]
        print '\n' + get_descendant_text(license) + '\n*/'

    def do_gtkdoc(self):
        for error in self.errors.getElementsByTagNameNS(NS_TP, 'error'):
            ns = error.parentNode.getAttribute('namespace')
            nick = error.getAttribute('name').replace(' ', '')
            enum = 'TP_ERROR_' + camelcase_to_upper(nick.replace('.', ''))
            print ' * @' + enum + ': ' + ns + '.' + nick + ':'
            print ' *     ' + get_docstring(error) + '    '

    def do_enumnames(self):
        for error in self.errors.getElementsByTagNameNS(NS_TP, 'error'):
            nick = error.getAttribute('name').replace(' ', '')
            enum = 'TP_ERROR_' + camelcase_to_upper(nick.replace('.', ''))
            print '    ' + enum + ','

    def do_get_type(self):
        print """
#include <glib-object.h>

G_BEGIN_DECLS

GType tp_error_get_type (void);

/**
 * TP_TYPE_ERROR:
 *
 * The GType of the Telepathy error enumeration.
 */
#define TP_TYPE_ERROR (tp_error_get_type())
"""

    def do_enum(self):
        print """\
/**
 * TpError:"""
        self.do_gtkdoc()
        print """\
 *
 * Enumerated type representing the Telepathy D-Bus errors.
 */
typedef enum {"""
        self.do_enumnames()
        print """\
} TpError;

G_END_DECLS"""

    def __call__(self):
        self.do_header()
        self.do_get_type()
        self.do_enum()

if __name__ == '__main__':
    argv = sys.argv[1:]
    Generator(xml.dom.minidom.parse(argv[0]))()



from unittest import *

class CheckResult (TestResult):

    successes = ()			# Immutable

    def addSuccess(self, test):
	if not self.successes:
	    self.successes = []
	self.successes.append(test)


class _CheckCase:

    def defaultTestResult(self):
	return CheckResult()


class CheckCase (_CheckCase, TestCase):
    pass


class FunctionCheckCase (_CheckCase, FunctionTestCase):
    pass
